/******************************************
 *       UNIQUE SEQUENCE IDENTIFIER       *
 *              Written By:               *
 *              Ross Walker               *
 *           Dept of Chemistry            *
 *            Imperial College            *
 *                 London                 *
 *                SW7 2AY                 *
 ******************************************/

/*The following defines numerical ID's for all amino acids
  in this way integer comparissons can be made in place of
  string comparissons allowing for greater processing speed
*/

/*COMPILE TIME OPTIONS
	-DUSE_LONG_LONG  = Uses long long in place of long to allow 64 bit offsets
*/

#ifdef USE_LONG_LONG
	typedef long long intlong;
#else
	typedef long intlong;
#endif



#define ALA 1 /*Alanine*/
#define ARG 2 /*Arginine*/
#define ASN 3 /*Asparagine*/
#define ASP 4 /*Apartic Acid*/
#define CYS 5 /*Cysteine*/
#define GLN 6 /*Glutamine*/
#define GLU 7 /*Glutamic Acid*/
#define GLY 8 /*Glycine*/
#define HIS 9 /*Histidine*/
#define ILE 10 /*Isoleucine*/
#define LEU 11 /*Leucine*/
#define LYS 12 /*Lysine*/
#define MET 13 /*Methionine*/
#define PHE 14 /*Phenylalanine*/
#define PRO 15 /*Proline*/
#define SER 16 /*Serine*/
#define THR 17 /*Threonine*/
#define TRP 18 /*Tryptophan*/
#define TYR 19 /*Tyrosine*/
#define VAL 20 /*Valine*/
#define amino_acid_max 20

/*Function Prototypes*/
void print_prog_info(void);
int process_command_line(int argc, char **argv);
int search_sequence_1(int *data, int *search, intlong data_length, intlong search_length);
void global_unlock(void);
void command_line_help(char *cmd_line_options[]);
void program_history(void);
void malloc_failure_char(char *routine, char *var_name, long bytes_requested);
void malloc_failure_int(char *routine, char *var_name, long bytes_requested);
int run_algorithm_1();
int run_algorithm_2();
void print_search_as_text(int *search, intlong search_length);
int read_comma_seq_list(char *filename);

/*externals*/
extern char *sequence_filename;
extern int algorithm;
extern intlong seq_data_length;
extern int *seq_data;
extern intlong start_search;
