/******************************************
 *       UNIQUE SEQUENCE IDENTIFIER       *
 *              Written By:               *
 *              Ross Walker               *
 *           Dept of Chemistry            *
 *            Imperial College            *
 *                 London                 *
 *                SW7 2AY                 *
 ******************************************/

/*This module contains the routines for all file
  access*/

#include <stdio.h>
#include <malloc.h>

#include "definitions.h"
  
int read_comma_seq_list(char *filename)
{
	/*error codes:
		0 = success
		1 = file not found
		2 = start_search>number of residues
	*/

	/*This routine expects a comma separated list or amino acids in the form
	  XXX,YYY,ZZZ,XXX,YYY,ZZZ etc...
	 */
	 
   FILE *fptr;
   char buffer; /*Used as temporary store for each data item read from input file*/
   intlong data_size=0;
   int triple_count=0;
   char test_string[3];

   /*Allocate an initial amoutn of memory*/
    seq_data=(int *)malloc((1)*sizeof(int));
   	if (seq_data==NULL)
   	{
   		malloc_failure_int("read_comma_seq_list", "seq_data", ((1)*(sizeof(int))));
   		return 1;
   	}

   
   if((fptr=fopen(filename,"rt"))==NULL)
   {
   		printf("FATAL ERROR - Input File %s not found\n",filename);
   		return 1;	/*we can't open the file*/
   }


   while(fscanf(fptr,"%c\n",&buffer)!=EOF)
   {
		data_size++;
		if (triple_count>=0)
			test_string[triple_count]=buffer;
		triple_count++;
		if (triple_count==3)
		{
			if ((test_string[0]=='A')&&(test_string[1]=='L')&&(test_string[2]=='A'))
				*(seq_data+seq_data_length)=ALA;
			else if ((test_string[0]=='A')&&(test_string[1]=='R')&&(test_string[2]=='G'))
				*(seq_data+seq_data_length)=ARG;
			else if ((test_string[0]=='A')&&(test_string[1]=='S')&&(test_string[2]=='N'))
				*(seq_data+seq_data_length)=ASN;
			else if ((test_string[0]=='A')&&(test_string[1]=='S')&&(test_string[2]=='P'))
				*(seq_data+seq_data_length)=ASP;
			else if ((test_string[0]=='C')&&(test_string[1]=='Y')&&(test_string[2]=='S'))
				*(seq_data+seq_data_length)=CYS;
			else if ((test_string[0]=='G')&&(test_string[1]=='L')&&(test_string[2]=='N'))
				*(seq_data+seq_data_length)=GLN;
			else if ((test_string[0]=='G')&&(test_string[1]=='L')&&(test_string[2]=='U'))
				*(seq_data+seq_data_length)=GLU;
			else if ((test_string[0]=='G')&&(test_string[1]=='L')&&(test_string[2]=='Y'))
				*(seq_data+seq_data_length)=GLY;
			else if ((test_string[0]=='H')&&(test_string[1]=='I')&&(test_string[2]=='S'))
				*(seq_data+seq_data_length)=HIS;
			else if ((test_string[0]=='I')&&(test_string[1]=='L')&&(test_string[2]=='E'))
				*(seq_data+seq_data_length)=ILE;
			else if ((test_string[0]=='L')&&(test_string[1]=='E')&&(test_string[2]=='U'))
				*(seq_data+seq_data_length)=LEU;
			else if ((test_string[0]=='L')&&(test_string[1]=='Y')&&(test_string[2]=='S'))
				*(seq_data+seq_data_length)=LYS;
			else if ((test_string[0]=='M')&&(test_string[1]=='E')&&(test_string[2]=='T'))
				*(seq_data+seq_data_length)=MET;
			else if ((test_string[0]=='P')&&(test_string[1]=='H')&&(test_string[2]=='E'))
				*(seq_data+seq_data_length)=PHE;	
			else if ((test_string[0]=='P')&&(test_string[1]=='R')&&(test_string[2]=='O'))
				*(seq_data+seq_data_length)=PRO;
			else if ((test_string[0]=='S')&&(test_string[1]=='E')&&(test_string[2]=='R'))
				*(seq_data+seq_data_length)=SER;
			else if ((test_string[0]=='T')&&(test_string[1]=='H')&&(test_string[2]=='R'))
				*(seq_data+seq_data_length)=THR;
			else if ((test_string[0]=='T')&&(test_string[1]=='R')&&(test_string[2]=='P'))
				*(seq_data+seq_data_length)=TRP;
			else if ((test_string[0]=='T')&&(test_string[1]=='Y')&&(test_string[2]=='R'))
				*(seq_data+seq_data_length)=TYR;
			else if ((test_string[0]=='V')&&(test_string[1]=='A')&&(test_string[2]=='L'))
				*(seq_data+seq_data_length)=VAL;
			else
			{
				printf("FATAL ERROR - Unknown data in input file: %c%c%c\n",test_string[0],test_string[1],test_string[2]);
				return 1;
			}
			
			seq_data_length++;
			/*allocate more memory*/
			seq_data=(int *)realloc(seq_data,(seq_data_length+1)*sizeof(int));
			if (seq_data==NULL)
			{
				malloc_failure_int("read_comma_seq_list", "seq_data", ((seq_data_length+1)*(sizeof(int))));
				return 1;
			}

			triple_count=-1; /*-1 as the next point should be a comma*/
		}
   }

   if (data_size==0)
   {
       printf("FATAL ERROR - Input File %s appears to be empty\n",filename);
       return 3;
   }
   
   printf("    Read a total of %ld residues from input file\n",seq_data_length);
     
   fclose(fptr);

   /*Check that start_search hasn't been set to larger than the number of residues*/
   if (start_search>seq_data_length)
   {
   		printf("FATAL ERROR - Specified start_loc of %ld is greater than\n",start_search);		
   		printf("              then number of residues in the data sequence %ld\n",seq_data_length);
   		return 2;
   
   }

	


	return 0;
}
