/******************************************
 *       UNIQUE SEQUENCE IDENTIFIER       *
 *              Written By:               *
 *              Ross Walker               *
 *           Dept of Chemistry            *
 *            Imperial College            *
 *                 London                 *
 *                SW7 2AY                 *
 ******************************************/

/*This module process's command line options*/

/*Options are:

	-i input  	Input file
	-a x  		Algorithm, methods 1 and 2 currently supported
*/

/*
   The function returns the following codes:
      0 = success
      1 = failure unknown option
      2 = failure too many options
      3 = allocation failure
      4 = User requested help
      5 = User requested program history
*/

#include <malloc.h>
#include <stdio.h>
#include <string.h>

#include "definitions.h"

int process_command_line(int argc, char **argv)
{
  int i;
  int retval;
  char *start_loc;
  /*Step 1, check to see if too many options have been specified */
  if (argc > 7)
  {
     return 2;
  }

  if (argc<2)
  {
      printf("\n  No command line options given - LOADING DEFAULTS\n");
      return 0;
  }

  if (!(strcmp(argv[1],"/?")) || !(strcmp(argv[1],"-?")) || !(strcmp(argv[1],"/help")) || !(strcmp(argv[1],"-help")) || !(strcmp(argv[1],"--help")))
  {
     /*User has requested help */
     return 4;
  }

  if (!(strcmp(argv[1],"/history")) || !(strcmp(argv[1],"/HISTORY")) || !(strcmp(argv[1],"/History")))
  {
     /*User has requested program history */
     return 5;
  }

  /*step 2, check each command line option to see if it is valid*/
  for (i=1; i<argc; i++)
  {
     /*Command line option 1 = -i energy_gap_filename*/
     if(!(strcmp(argv[i],"-i")))
     {
        /*option is for sequence filename - This is stored in the global pointer
         *sequence_filename*/
        i++; /*Increment i to get next command line option */

		if (i >= argc) /*User didn't specify anything after the -i */
           return 1;

        /* *sequence_filename was already allocated by set_default_options()
           so we have to use realloc */
        sequence_filename = (char *)realloc(sequence_filename,((strlen(argv[i])+1)*sizeof(char)));
        if (sequence_filename == NULL)
        {
           malloc_failure_char("process_command_line", "sequence_filename", ((strlen(argv[i])+1)*sizeof(char)));
           return 3;
        }
        strcpy(sequence_filename,argv[i]);
 
    }
    else if(!(strcmp(argv[i],"-a")))
    {        /*Command line option 2 = -a algorithm*/
         i++; /*Increment i to get next command line option */

      	 if (i >= argc) /*User didn't specify anything after the -a */
          	return 1;
		
		 if (*argv[i]==(char)49) /*ascii 49 = 1*/
			 algorithm = 1;
		 else if (*argv[i]==(char)50) /*ascii 50 = 2*/
		 	 algorithm = 2;
		 else
		 	return 1;

    }
    else if(!(strcmp(argv[i],"-s")))
    {        /*Command line option 3 = -s start_search*/
         i++; /*Increment i to get next command line option */

      	 if (i >= argc) /*User didn't specify anything after the -s */
          	return 1;
		
	    start_loc = (char *)malloc(((strlen(argv[i])+1)*sizeof(char)));
        if (start_loc == NULL)
        {
           malloc_failure_char("process_command_line", "start_loc", ((strlen(argv[i])+1)*sizeof(char)));
           return 3;
        }
        strcpy(start_loc,argv[i]);
        /*Now scan the sting for a number*/
        retval=sscanf(start_loc,"%ld",&start_search);
        if (retval!=1)
        {
        	printf("Invalid option for start location - Ignored\n");
        	printf("Search will be started from length = 1\n");
        	start_search=1;
        }
        /*Unfortunately we haven't scanned the input file yet so I cannot check that
          start_search<=number of residues in file so we will need to check this
          later when we read the file*/
        
        free(start_loc);


     }
     else
     {
         return 1; /*Unknown command line option*/
     }

  }
  return 0;
}

