/******************************************
 *       UNIQUE SEQUENCE IDENTIFIER       *
 *              Written By:               *
 *              Ross Walker               *
 *           Dept of Chemistry            *
 *            Imperial College            *
 *                 London                 *
 *                SW7 2AY                 *
 ******************************************/

#include <stdio.h>
#include <malloc.h>

#include "definitions.h"

int run_algorithm_1()
{
	/*Return codes:
		0 = success - unique found
		1 = Failure
	*/
	int i=0;
	int *search;
	intlong search_length=start_search-1;
	intlong current_power=0;
	int power_complete_flag=0;
	int loop_exit=0;
	int amino_acid;
	int uniques_found=0;
	
	printf("    Processing Sequence using Algorithm 1\n\n");
	
	/*Step 1, read in sequence*/
	
	i=read_comma_seq_list(sequence_filename);
	if (i!=0)
		return 1;

	/*Allocate memory for *search*/
	/*Maximum search length is the length of the input file so we will allocate enough
	  memory for this, I know this is wasteful but it is faster*/
	search=(int *)malloc((seq_data_length+1)*sizeof(int));
	if (search==NULL)
	{
		malloc_failure_int("run_algorithm_1", "search", ((search_length+1)*(sizeof(int))));
		return 1;
	}
	/*initialise the entire array*/
	for (i=0;i<=seq_data_length;i++)
		*(search+i)=1;

	/*Now we construct our search string and pass it to search_sequence_1*/
	while (loop_exit==0)
	{
		/*This will continually loop until we deliberately break out with loop_exit=1*/
		for (amino_acid=1;amino_acid<=amino_acid_max;amino_acid++)
		{
			/*This will loop through for all amino_acids from 1 to amino_acid_max*/
			/*On the last amino acid in our chain each time e.g xyz it scales z from 1 to amino_acid_max*/
			*(search)=amino_acid;	
			i=search_sequence_1(seq_data, search, seq_data_length, search_length);
			if (i==1)
			{
				uniques_found++;
				if (uniques_found==1)
				{
						printf("\n      Unique Sequence Identified - Length = %ld\n",search_length+1);
				}
				printf("      Sequence %d ID: ",uniques_found);
				print_search_as_text(search, search_length);
				printf("\n");
				
			}
		}

		/*for (i=1;i<=seq_data_length;i++)
			printf("%ld ",*(search+i));
		printf("\n");*/

		for (current_power=0;(current_power<=search_length) && (loop_exit==0);current_power++)
		{
			/*Check to see if any have reached 20 and if so increment by 1,
			  if current_power=search_length and it equals 20 then increment
			  search_length by 1 and set value to 1*/
	 
			 if (*(search+current_power)>=20)
			 {
				if (current_power==search_length)
				{
					/*increment search_length*/
					*(search+search_length)=1;
					search_length++;
					*(search+search_length)=1;
					power_complete_flag=1;
					break;
				}
				else
				{
					*(search+current_power)=1;
					(*(search+current_power+1))++;
					if (*(search+current_power+1)<=20)
						break; /*Only break if we haven't pushed the next power too high*/
				}
			}
			else
			{
					(*(search+current_power))++;
					break;
			}	 
		}	

		if (power_complete_flag==1)
		{
			if (uniques_found>=1)
				loop_exit=1;
			power_complete_flag=0;
			printf("      Completed Search of Amino Acid Chain Length: %ld\n",search_length);
		}
		if (search_length>=seq_data_length)
		{
			printf("Error Detected -> Program Halted\n");
			printf("search_length of %ld is greater than seq_data_length of %ld\n",search_length+1,seq_data_length);
			printf("Confused... This should never happen\n");
			return 1;
		}
	}
	
	
	free(search);
	return 0;
}
