/******************************************
 *       UNIQUE SEQUENCE IDENTIFIER       *
 *              Written By:               *
 *              Ross Walker               *
 *           Dept of Chemistry            *
 *            Imperial College            *
 *                 London                 *
 *                SW7 2AY                 *
 ******************************************/

/* This module contains the routines that
   search through the sequence looking for
   matches between search string and data
   string.
*/

#include "definitions.h"

int search_sequence_1(int *data, int *search, intlong data_length, intlong search_length)
{
	/*This routine is algorithm 1*/
	/*The simplest but most cpu intense algorithm available*/
	
	/*We receive the sequence in *data of length data_length
	  and our search string in *search of length search_length
	*/
	
	/*Return codes:
		0 = Found no occurance of *search in *data
		1 = Found only 1 occurance = success
		2 = Found more than 1 occurance = no unique
	*/
	
	int found=0;
	intlong data_pos;
	intlong search_pos;
	int match=0;
	
	for (data_pos=0;data_pos<data_length;data_pos++)
	{
		/*This loop slides us along the data array 1 position at a time*/
		
		for (search_pos=0;search_pos<=search_length;search_pos++)
		{
			/*This loop takes the data and checks if each value in
			  search data matches
			*/
			if (*(data+data_pos+search_pos)==*(search+search_pos))
				match=1;
			else
			{
				match=0;
				break; /*Break out the loop as no match*/
			}
		}
		/*if we had a match we set our found to true - if it is already true we
		  have found multiple copies so beak out*/
		if (match==1)
		{
			if (found!=1)
				found=1;
			else
			{
				found=2;
				break;
			}
		}
		
		/*reset variables for next loop*/
		match=0;
	}
	
	if (found==0)
		return 0; /*No match*/
	else if (found==1)
		return 1; /*unique match*/
	else
		return 2; /*Multiple matches*/

}

